# -*- coding: utf-8 -*-
"""
An engine that uses presence detection to keep track of which minions
have been recently connected and remove their keys if they have not been
connected for a certain period of time.

Requires that the :conf_master:`minion_data_cache` option be enabled.

.. versionadded: 2017.7.0

:configuration:

    Example configuration:

    .. code-block:: yaml

        engines:
          - stalekey:
              interval: 3600
              expire: 86400

"""
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import time

# Import salt libs
import salt.config
import salt.key
import salt.utils.files
import salt.utils.minions
import salt.utils.msgpack
import salt.wheel

# Import 3rd-party libs
from salt.ext import six

log = logging.getLogger(__name__)


def __virtual__():
    if not __opts__.get("minion_data_cache"):
        return (False, "stalekey engine requires minion_data_cache to be enabled")
    return True


def _get_keys():
    keys = salt.key.get_key(__opts__)
    minions = keys.all_keys()
    return minions["minions"]


def start(interval=3600, expire=604800):
    ck = salt.utils.minions.CkMinions(__opts__)
    presence_file = "{0}/presence.p".format(__opts__["cachedir"])
    wheel = salt.wheel.WheelClient(__opts__)

    while True:
        log.debug("Checking for present minions")
        minions = {}
        if os.path.exists(presence_file):
            try:
                with salt.utils.files.fopen(presence_file, "r") as f:
                    minions = salt.utils.msgpack.load(f)
            except IOError as e:
                log.error("Could not open presence file %s: %s", presence_file, e)
                time.sleep(interval)
                continue

        minion_keys = _get_keys()
        now = time.time()
        present = ck.connected_ids()

        # For our existing keys, check which are present
        for m in minion_keys:
            # If we have a key that's not in the presence file, it may be a new minion
            # It could also mean this is the first time this engine is running and no
            # presence file was found
            if m not in minions:
                minions[m] = now
            elif m in present:
                minions[m] = now

        log.debug("Finished checking for present minions")
        # Delete old keys
        stale_keys = []
        for m, seen in six.iteritems(minions):
            if now - expire > seen:
                stale_keys.append(m)

        if stale_keys:
            for k in stale_keys:
                log.info("Removing stale key for %s", k)
            wheel.cmd("key.delete", stale_keys)
            del minions[k]

        try:
            with salt.utils.files.fopen(presence_file, "w") as f:
                salt.utils.msgpack.dump(minions, f)
        except IOError as e:
            log.error("Could not write to presence file %s: %s", presence_file, e)
        time.sleep(interval)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import re
import sys
import textwrap

import salt.utils.stringutils

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import builtins, range  # pylint: disable=redefined-builtin

# Import Salt libs
from tests.support.mock import patch
from tests.support.unit import LOREM_IPSUM, TestCase

UNICODE = "中国語 (繁体)"
STR = BYTES = UNICODE.encode("utf-8")
# This is an example of a unicode string with й constructed using two separate
# code points. Do not modify it.
EGGS = "\u044f\u0438\u0306\u0446\u0430"

LATIN1_UNICODE = "räksmörgås"
LATIN1_BYTES = LATIN1_UNICODE.encode("latin-1")

DOUBLE_TXT = """\
# set variable identifying the chroot you work in (used in the prompt below)
if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi
"""

SINGLE_TXT = """\
# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi
"""

SINGLE_DOUBLE_TXT = """\
# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi

# set variable identifying the chroot you work in (used in the prompt below)
if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi
"""

SINGLE_DOUBLE_SAME_LINE_TXT = """\
# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r "/etc/debian_chroot" ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi
"""

MATCH = """\
# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi


# set variable identifying the chroot you work in (used in the prompt below)
if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi


# set variable identifying the chroot you work in (used in the prompt below)
if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi


# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r /etc/debian_chroot ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi


# set variable identifying the chroot you work in (used in the prompt below)
if [ -z '$debian_chroot' ] && [ -r "/etc/debian_chroot" ]; then
    debian_chroot=$(cat /etc/debian_chroot)
fi
"""


class TestBuildWhitespaceRegex(TestCase):
    def test_single_quotes(self):
        regex = salt.utils.stringutils.build_whitespace_split_regex(SINGLE_TXT)
        self.assertTrue(re.search(regex, MATCH))

    def test_double_quotes(self):
        regex = salt.utils.stringutils.build_whitespace_split_regex(DOUBLE_TXT)
        self.assertTrue(re.search(regex, MATCH))

    def test_single_and_double_quotes(self):
        regex = salt.utils.stringutils.build_whitespace_split_regex(SINGLE_DOUBLE_TXT)
        self.assertTrue(re.search(regex, MATCH))

    def test_issue_2227(self):
        regex = salt.utils.stringutils.build_whitespace_split_regex(
            SINGLE_DOUBLE_SAME_LINE_TXT
        )
        self.assertTrue(re.search(regex, MATCH))


class StringutilsTestCase(TestCase):
    def test_contains_whitespace(self):
        does_contain_whitespace = "A brown fox jumped over the red hen."
        does_not_contain_whitespace = "Abrownfoxjumpedovertheredhen."

        self.assertTrue(
            salt.utils.stringutils.contains_whitespace(does_contain_whitespace)
        )
        self.assertFalse(
            salt.utils.stringutils.contains_whitespace(does_not_contain_whitespace)
        )

    def test_to_num(self):
        self.assertEqual(7, salt.utils.stringutils.to_num("7"))
        self.assertIsInstance(salt.utils.stringutils.to_num("7"), int)
        self.assertEqual(7, salt.utils.stringutils.to_num("7.0"))
        self.assertIsInstance(salt.utils.stringutils.to_num("7.0"), float)
        self.assertEqual(salt.utils.stringutils.to_num("Seven"), "Seven")
        self.assertIsInstance(salt.utils.stringutils.to_num("Seven"), six.text_type)

    def test_to_none(self):
        self.assertIsNone(salt.utils.stringutils.to_none(""))
        self.assertIsNone(salt.utils.stringutils.to_none("  "))
        # Ensure that we do not inadvertently convert certain strings or 0 to None
        self.assertIsNotNone(salt.utils.stringutils.to_none("None"))
        self.assertIsNotNone(salt.utils.stringutils.to_none(0))

    def test_is_binary(self):
        self.assertFalse(salt.utils.stringutils.is_binary(LOREM_IPSUM))
        # Also test bytestring
        self.assertFalse(
            salt.utils.stringutils.is_binary(
                salt.utils.stringutils.is_binary(LOREM_IPSUM)
            )
        )

        zero_str = "{0}{1}".format(LOREM_IPSUM, "\0")
        self.assertTrue(salt.utils.stringutils.is_binary(zero_str))
        # Also test bytestring
        self.assertTrue(
            salt.utils.stringutils.is_binary(salt.utils.stringutils.to_bytes(zero_str))
        )

        # To to ensure safe exit if str passed doesn't evaluate to True
        self.assertFalse(salt.utils.stringutils.is_binary(""))
        self.assertFalse(salt.utils.stringutils.is_binary(b""))

        nontext = 3 * (
            "".join([chr(x) for x in range(1, 32) if x not in (8, 9, 10, 12, 13)])
        )
        almost_bin_str = "{0}{1}".format(LOREM_IPSUM[:100], nontext[:42])
        self.assertFalse(salt.utils.stringutils.is_binary(almost_bin_str))
        # Also test bytestring
        self.assertFalse(
            salt.utils.stringutils.is_binary(
                salt.utils.stringutils.to_bytes(almost_bin_str)
            )
        )

        bin_str = almost_bin_str + "\x01"
        self.assertTrue(salt.utils.stringutils.is_binary(bin_str))
        # Also test bytestring
        self.assertTrue(
            salt.utils.stringutils.is_binary(salt.utils.stringutils.to_bytes(bin_str))
        )

    def test_to_str(self):
        for x in (123, (1, 2, 3), [1, 2, 3], {1: 23}, None):
            self.assertRaises(TypeError, salt.utils.stringutils.to_str, x)
        if six.PY3:
            self.assertEqual(salt.utils.stringutils.to_str("plugh"), "plugh")
            self.assertEqual(salt.utils.stringutils.to_str("áéíóúý", "utf-8"), "áéíóúý")
            self.assertEqual(salt.utils.stringutils.to_str(BYTES, "utf-8"), UNICODE)
            self.assertEqual(
                salt.utils.stringutils.to_str(bytearray(BYTES), "utf-8"), UNICODE
            )
            # Test situation when a minion returns incorrect utf-8 string because of... million reasons
            ut2 = b"\x9c"
            self.assertRaises(
                UnicodeDecodeError, salt.utils.stringutils.to_str, ut2, "utf-8"
            )
            self.assertEqual(
                salt.utils.stringutils.to_str(ut2, "utf-8", "replace"), "\ufffd"
            )
            self.assertRaises(
                UnicodeDecodeError,
                salt.utils.stringutils.to_str,
                bytearray(ut2),
                "utf-8",
            )
            self.assertEqual(
                salt.utils.stringutils.to_str(bytearray(ut2), "utf-8", "replace"),
                "\ufffd",
            )
        else:
            self.assertEqual(
                salt.utils.stringutils.to_str("plugh"), str("plugh")
            )  # future lint: disable=blacklisted-function
            self.assertEqual(
                salt.utils.stringutils.to_str("áéíóúý", "utf-8"),
                "áéíóúý".encode("utf-8"),
            )
            self.assertEqual(salt.utils.stringutils.to_str(UNICODE, "utf-8"), STR)
            self.assertEqual(
                salt.utils.stringutils.to_str(bytearray(STR), "utf-8"), STR
            )

            # Test utf-8 fallback with Windows default codepage
            with patch.object(builtins, "__salt_system_encoding__", "CP1252"):
                self.assertEqual(
                    salt.utils.stringutils.to_str("Ψ"), "Ψ".encode("utf-8")
                )

    def test_to_bytes(self):
        for x in (123, (1, 2, 3), [1, 2, 3], {1: 23}, None):
            self.assertRaises(TypeError, salt.utils.stringutils.to_bytes, x)
        if six.PY3:
            self.assertEqual(salt.utils.stringutils.to_bytes("xyzzy"), b"xyzzy")
            self.assertEqual(salt.utils.stringutils.to_bytes(BYTES), BYTES)
            self.assertEqual(salt.utils.stringutils.to_bytes(bytearray(BYTES)), BYTES)
            self.assertEqual(salt.utils.stringutils.to_bytes(UNICODE, "utf-8"), BYTES)

            # Test utf-8 fallback with ascii default encoding
            with patch.object(builtins, "__salt_system_encoding__", "ascii"):
                self.assertEqual(salt.utils.stringutils.to_bytes("Ψ"), b"\xce\xa8")
        else:
            self.assertEqual(salt.utils.stringutils.to_bytes("xyzzy"), "xyzzy")
            self.assertEqual(salt.utils.stringutils.to_bytes(BYTES), BYTES)
            self.assertEqual(salt.utils.stringutils.to_bytes(bytearray(BYTES)), BYTES)
            self.assertEqual(salt.utils.stringutils.to_bytes(UNICODE, "utf-8"), BYTES)

    def test_to_unicode(self):
        self.assertEqual(
            salt.utils.stringutils.to_unicode(EGGS, normalize=True), "яйца"
        )
        self.assertNotEqual(
            salt.utils.stringutils.to_unicode(EGGS, normalize=False), "яйца"
        )

        self.assertEqual(
            salt.utils.stringutils.to_unicode(LATIN1_BYTES, encoding="latin-1"),
            LATIN1_UNICODE,
        )

        if six.PY3:
            self.assertEqual(salt.utils.stringutils.to_unicode("plugh"), "plugh")
            self.assertEqual(salt.utils.stringutils.to_unicode("áéíóúý"), "áéíóúý")
            self.assertEqual(salt.utils.stringutils.to_unicode(BYTES, "utf-8"), UNICODE)
            self.assertEqual(
                salt.utils.stringutils.to_unicode(bytearray(BYTES), "utf-8"), UNICODE
            )
        else:
            self.assertEqual(
                salt.utils.stringutils.to_unicode(str("xyzzy"), "utf-8"), "xyzzy"
            )  # future lint: disable=blacklisted-function
            self.assertEqual(salt.utils.stringutils.to_unicode(BYTES, "utf-8"), UNICODE)

            # Test that unicode chars are decoded properly even when using
            # locales which are not UTF-8 compatible
            with patch.object(builtins, "__salt_system_encoding__", "ascii"):
                self.assertEqual(
                    salt.utils.stringutils.to_unicode("Ψ".encode("utf-8")), "Ψ"
                )
            with patch.object(builtins, "__salt_system_encoding__", "CP1252"):
                self.assertEqual(
                    salt.utils.stringutils.to_unicode("Ψ".encode("utf-8")), "Ψ"
                )

    def test_to_unicode_multi_encoding(self):
        result = salt.utils.stringutils.to_unicode(
            LATIN1_BYTES, encoding=("utf-8", "latin1")
        )
        assert result == LATIN1_UNICODE

    def test_build_whitespace_split_regex(self):
        # With 3.7+,  re.escape only escapes special characters, no longer
        # escaping all characters other than ASCII letters, numbers and
        # underscores.  This includes commas.
        if sys.version_info >= (3, 7):
            expected_regex = (
                "(?m)^(?:[\\s]+)?Lorem(?:[\\s]+)?ipsum(?:[\\s]+)?dolor(?:[\\s]+)?sit(?:[\\s]+)?amet,"
                "(?:[\\s]+)?$"
            )
        else:
            expected_regex = (
                "(?m)^(?:[\\s]+)?Lorem(?:[\\s]+)?ipsum(?:[\\s]+)?dolor(?:[\\s]+)?sit(?:[\\s]+)?amet\\,"
                "(?:[\\s]+)?$"
            )
        ret = salt.utils.stringutils.build_whitespace_split_regex(
            " ".join(LOREM_IPSUM.split()[:5])
        )
        self.assertEqual(ret, expected_regex)

    def test_get_context(self):
        expected_context = textwrap.dedent(
            """\
            ---
            Lorem ipsum dolor sit amet, consectetur adipiscing elit. Quisque eget urna a arcu lacinia sagittis.
            Sed scelerisque, lacus eget malesuada vestibulum, justo diam facilisis tortor, in sodales dolor
            [...]
            ---"""
        )
        ret = salt.utils.stringutils.get_context(LOREM_IPSUM, 1, num_lines=1)
        self.assertEqual(ret, expected_context)

    def test_get_context_has_enough_context(self):
        template = "1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf"
        context = salt.utils.stringutils.get_context(template, 8)
        expected = "---\n[...]\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\n[...]\n---"
        self.assertEqual(expected, context)

    def test_get_context_at_top_of_file(self):
        template = "1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf"
        context = salt.utils.stringutils.get_context(template, 1)
        expected = "---\n1\n2\n3\n4\n5\n6\n[...]\n---"
        self.assertEqual(expected, context)

    def test_get_context_at_bottom_of_file(self):
        template = "1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf"
        context = salt.utils.stringutils.get_context(template, 15)
        expected = "---\n[...]\na\nb\nc\nd\ne\nf\n---"
        self.assertEqual(expected, context)

    def test_get_context_2_context_lines(self):
        template = "1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf"
        context = salt.utils.stringutils.get_context(template, 8, num_lines=2)
        expected = "---\n[...]\n6\n7\n8\n9\na\n[...]\n---"
        self.assertEqual(expected, context)

    def test_get_context_with_marker(self):
        template = "1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf"
        context = salt.utils.stringutils.get_context(
            template, 8, num_lines=2, marker=" <---"
        )
        expected = "---\n[...]\n6\n7\n8 <---\n9\na\n[...]\n---"
        self.assertEqual(expected, context)

    def test_expr_match(self):
        val = "foo/bar/baz"
        # Exact match
        self.assertTrue(salt.utils.stringutils.expr_match(val, val))
        # Glob match
        self.assertTrue(salt.utils.stringutils.expr_match(val, "foo/*/baz"))
        # Glob non-match
        self.assertFalse(salt.utils.stringutils.expr_match(val, "foo/*/bar"))
        # Regex match
        self.assertTrue(salt.utils.stringutils.expr_match(val, r"foo/\w+/baz"))
        # Regex non-match
        self.assertFalse(salt.utils.stringutils.expr_match(val, r"foo/\w/baz"))

    def test_check_whitelist_blacklist(self):
        """
        Ensure that whitelist matching works on both PY2 and PY3
        """
        whitelist = ["one/two/three", r"web[0-9]"]
        blacklist = ["four/five/six", r"web[5-9]"]

        # Tests with string whitelist/blacklist
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=whitelist[1], blacklist=None,
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=whitelist[1], blacklist=[],
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=whitelist[1], blacklist=None,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=whitelist[1], blacklist=[],
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=None, blacklist=blacklist[1],
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=[], blacklist=blacklist[1],
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=None, blacklist=blacklist[1],
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=[], blacklist=blacklist[1],
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=whitelist[1], blacklist=blacklist[1],
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web4", whitelist=whitelist[1], blacklist=blacklist[1],
            )
        )

        # Tests with list whitelist/blacklist
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=whitelist, blacklist=None,
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=whitelist, blacklist=[],
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=whitelist, blacklist=None,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=whitelist, blacklist=[],
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=None, blacklist=blacklist,
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=[], blacklist=blacklist,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=None, blacklist=blacklist,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=[], blacklist=blacklist,
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=whitelist, blacklist=blacklist,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web4", whitelist=whitelist, blacklist=blacklist,
            )
        )

        # Tests with set whitelist/blacklist
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=set(whitelist), blacklist=None,
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_one", whitelist=set(whitelist), blacklist=set(),
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=set(whitelist), blacklist=None,
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web1", whitelist=set(whitelist), blacklist=set(),
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=None, blacklist=set(blacklist),
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=set(), blacklist=set(blacklist),
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=None, blacklist=set(blacklist),
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web_five", whitelist=set(), blacklist=set(blacklist),
            )
        )
        self.assertFalse(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web5", whitelist=set(whitelist), blacklist=set(blacklist),
            )
        )
        self.assertTrue(
            salt.utils.stringutils.check_whitelist_blacklist(
                "web4", whitelist=set(whitelist), blacklist=set(blacklist),
            )
        )

        # Test with invalid type for whitelist/blacklist
        self.assertRaises(
            TypeError,
            salt.utils.stringutils.check_whitelist_blacklist,
            "foo",
            whitelist=123,
        )
        self.assertRaises(
            TypeError,
            salt.utils.stringutils.check_whitelist_blacklist,
            "foo",
            blacklist=123,
        )

    def test_check_include_exclude_empty(self):
        self.assertTrue(salt.utils.stringutils.check_include_exclude("/some/test"))

    def test_check_include_exclude_exclude(self):
        self.assertFalse(
            salt.utils.stringutils.check_include_exclude("/some/test", None, "*test*")
        )

    def test_check_include_exclude_exclude_list(self):
        self.assertFalse(
            salt.utils.stringutils.check_include_exclude("/some/test", None, ["*test"])
        )

    def test_check_include_exclude_exclude_include(self):
        self.assertTrue(
            salt.utils.stringutils.check_include_exclude(
                "/some/test", "*test*", "/some/"
            )
        )

    def test_check_include_exclude_regex(self):
        self.assertFalse(
            salt.utils.stringutils.check_include_exclude(
                "/some/test", None, "E@/some/(test|other)"
            )
        )



# -*- coding: utf-8 -*-

"""
Manage Chocolatey package installs
.. versionadded:: 2016.3.0

.. note::
    Chocolatey pulls data from the Chocolatey internet database to determine
    current versions, find available versions, etc. This is normally a slow
    operation and may be optimized by specifying a local, smaller chocolatey
    repo.
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.utils.data
import salt.utils.versions
from salt.exceptions import SaltInvocationError


def __virtual__():
    """
    Load only if chocolatey is loaded
    """
    if "chocolatey.install" in __salt__:
        return "chocolatey"
    return (False, "chocolatey module could not be loaded")


def installed(
    name,
    version=None,
    source=None,
    force=False,
    pre_versions=False,
    install_args=None,
    override_args=False,
    force_x86=False,
    package_args=None,
    allow_multiple=False,
):
    """
    Installs a package if not already installed

    Args:

        name (str):
            The name of the package to be installed. Required.

        version (str):
            Install a specific version of the package. Defaults to latest
            version. If the version is different to the one installed then the
            specified version will be installed. Default is None.

        source (str):
            Chocolatey repository (directory, share or remote URL, feed).
            Defaults to the official Chocolatey feed. Default is None.

        force (bool):
            Reinstall the current version of an existing package. Do not use
            with ``allow_multiple``. Default is False.

        pre_versions (bool):
            Include pre-release packages. Default is False.

        install_args (str):
            Install arguments you want to pass to the installation process, i.e
            product key or feature list. Default is None.

        override_args (bool):
            Set to True if you want to override the original install arguments
            (for the native installer) in the package and use your own. When
            this is set to False install_args will be appended to the end of the
            default arguments. Default is False.

        force_x86 (bool):
            Force x86 (32bit) installation on 64 bit systems. Default is False.

        package_args (str):
            Arguments you want to pass to the package. Default is None.

        allow_multiple (bool):
            Allow mulitiple versions of the package to be installed. Do not use
            with ``force``. Does not work with all packages. Default is False.

            .. versionadded:: 2017.7.0

    .. code-block:: yaml

        Installsomepackage:
          chocolatey.installed:
            - name: packagename
            - version: '12.04'
            - source: 'mychocolatey/source'
            - force: True
    """
    if force and allow_multiple:
        raise SaltInvocationError(
            "Cannot use 'force' in conjunction with 'allow_multiple'"
        )

    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    # Get list of currently installed packages
    pre_install = __salt__["chocolatey.list"](local_only=True)

    # Determine action
    # Package not installed
    if name.lower() not in [package.lower() for package in pre_install.keys()]:
        if version:
            ret["changes"] = {name: "Version {0} will be installed".format(version)}
        else:
            ret["changes"] = {name: "Latest version will be installed"}

    # Package installed
    else:
        version_info = __salt__["chocolatey.version"](
            name=name, check_remote=True, source=source
        )

        full_name = name
        for pkg in version_info:
            if name.lower() == pkg.lower():
                full_name = pkg

        installed_version = version_info[full_name]["installed"][0]

        if version:
            if salt.utils.versions.compare(
                ver1=installed_version, oper="==", ver2=version
            ):
                if force:
                    ret["changes"] = {
                        name: "Version {0} will be reinstalled".format(version)
                    }
                    ret["comment"] = "Reinstall {0} {1}".format(full_name, version)
                else:
                    ret["comment"] = "{0} {1} is already installed".format(
                        name, version
                    )
                    if __opts__["test"]:
                        ret["result"] = None
                    return ret
            else:
                if allow_multiple:
                    ret["changes"] = {
                        name: "Version {0} will be installed side by side with "
                        "Version {1} if supported".format(version, installed_version)
                    }
                    ret["comment"] = "Install {0} {1} side-by-side with {0} {2}".format(
                        full_name, version, installed_version
                    )
                else:
                    ret["changes"] = {
                        name: "Version {0} will be installed over Version {1}".format(
                            version, installed_version
                        )
                    }
                    ret["comment"] = "Install {0} {1} over {0} {2}".format(
                        full_name, version, installed_version
                    )
                    force = True
        else:
            version = installed_version
            if force:
                ret["changes"] = {
                    name: "Version {0} will be reinstalled".format(version)
                }
                ret["comment"] = "Reinstall {0} {1}".format(full_name, version)
            else:
                ret["comment"] = "{0} {1} is already installed".format(name, version)
                if __opts__["test"]:
                    ret["result"] = None
                return ret

    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "The installation was tested"
        return ret

    # Install the package
    result = __salt__["chocolatey.install"](
        name=name,
        version=version,
        source=source,
        force=force,
        pre_versions=pre_versions,
        install_args=install_args,
        override_args=override_args,
        force_x86=force_x86,
        package_args=package_args,
        allow_multiple=allow_multiple,
    )

    if "Running chocolatey failed" not in result:
        ret["result"] = True
    else:
        ret["result"] = False

    if not ret["result"]:
        ret["comment"] = "Failed to install the package {0}".format(name)

    # Get list of installed packages after 'chocolatey.install'
    post_install = __salt__["chocolatey.list"](local_only=True)

    ret["changes"] = salt.utils.data.compare_dicts(pre_install, post_install)

    return ret


def uninstalled(name, version=None, uninstall_args=None, override_args=False):
    """
    Uninstalls a package

    name
      The name of the package to be uninstalled

    version
      Uninstalls a specific version of the package. Defaults to latest
      version installed.

    uninstall_args
      A list of uninstall arguments you want to pass to the uninstallation
      process i.e product key or feature list

    override_args
      Set to true if you want to override the original uninstall arguments (
      for the native uninstaller)in the package and use your own.
      When this is set to False uninstall_args will be appended to the end of
      the default arguments

    .. code-block:: yaml

      Removemypackage:
        chocolatey.uninstalled:
          - name: mypackage
          - version: '21.5'

    """

    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    # Get list of currently installed packages
    pre_uninstall = __salt__["chocolatey.list"](local_only=True)

    # Determine if package is installed
    if name.lower() in [package.lower() for package in pre_uninstall.keys()]:
        try:
            ret["changes"] = {
                name: "{0} version {1} will be removed".format(
                    name, pre_uninstall[name][0]
                )
            }
        except KeyError:
            ret["changes"] = {name: "{0} will be removed".format(name)}
    else:
        ret["comment"] = "The package {0} is not installed".format(name)
        return ret

    if __opts__["test"]:
        ret["result"] = None
        ret["comment"] = "The uninstall was tested"
        return ret

    # Uninstall the package
    result = __salt__["chocolatey.uninstall"](
        name, version, uninstall_args, override_args
    )

    if "Running chocolatey failed" not in result:
        ret["result"] = True
    else:
        ret["result"] = False

    if not ret["result"]:
        ret["comment"] = "Failed to uninstall the package {0}".format(name)

    # Get list of installed packages after 'chocolatey.uninstall'
    post_uninstall = __salt__["chocolatey.list"](local_only=True)

    ret["changes"] = salt.utils.data.compare_dicts(pre_uninstall, post_uninstall)

    return ret


def upgraded(
    name,
    version=None,
    source=None,
    force=False,
    pre_versions=False,
    install_args=None,
    override_args=False,
    force_x86=False,
    package_args=None,
):
    """
    Upgrades a package. Will install the package if not installed.

    .. versionadded:: 2018.3.0

    Args:

        name (str):
            The name of the package to be installed. Required.

        version (str):
            Install a specific version of the package. Defaults to latest
            version. If the version is greater than the one installed then the
            specified version will be installed. Default is ``None``.

        source (str):
            Chocolatey repository (directory, share or remote URL, feed).
            Defaults to the official Chocolatey feed. Default is ``None``.

        force (bool):
            ``True`` will reinstall an existing package with the same version.
            Default is ``False``.

        pre_versions (bool):
            ``True`` will nclude pre-release packages. Default is ``False``.

        install_args (str):
            Install arguments you want to pass to the installation process, i.e
            product key or feature list. Default is ``None``.

        override_args (bool):
            ``True`` will override the original install arguments (for the
            native installer) in the package and use those specified in
            ``install_args``. ``False`` will append install_args to the end of
            the default arguments. Default is ``False``.

        force_x86 (bool):
            ``True`` forces 32bit installation on 64 bit systems. Default is
            ``False``.

        package_args (str):
            Arguments you want to pass to the package. Default is ``None``.

    .. code-block:: yaml

        upgrade_some_package:
          chocolatey.upgraded:
            - name: packagename
            - version: '12.04'
            - source: 'mychocolatey/source'
    """
    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    # Get list of currently installed packages
    pre_install = __salt__["chocolatey.list"](local_only=True)

    # Determine if there are changes
    # Package not installed
    if name.lower() not in [package.lower() for package in pre_install.keys()]:
        if version:
            ret["changes"][name] = "Version {0} will be installed".format(version)
            ret["comment"] = "Install version {0}".format(version)
        else:
            ret["changes"][name] = "Latest version will be installed"
            ret["comment"] = "Install latest version"

    # Package installed
    else:
        version_info = __salt__["chocolatey.version"](name, check_remote=True)

        # Get the actual full name out of version_info
        full_name = name
        for pkg in version_info:
            if name.lower() == pkg.lower():
                full_name = pkg

        installed_version = version_info[full_name]["installed"][0]

        # If version is not passed, use available... if available is available
        if not version:
            if "available" in version_info[full_name]:
                version = version_info[full_name]["available"][0]

        if version:
            # If installed version and new version are the same
            if salt.utils.versions.compare(
                ver1=installed_version, oper="==", ver2=version
            ):
                if force:
                    ret["changes"][name] = "Version {0} will be reinstalled".format(
                        version
                    )
                    ret["comment"] = "Reinstall {0} {1}".format(full_name, version)
                else:
                    ret["comment"] = "{0} {1} is already installed".format(
                        name, installed_version
                    )
            else:
                # If installed version is older than new version
                if salt.utils.versions.compare(
                    ver1=installed_version, oper="<", ver2=version
                ):
                    ret["changes"][
                        name
                    ] = "Version {0} will be upgraded to Version {1}".format(
                        installed_version, version
                    )
                    ret["comment"] = "Upgrade {0} {1} to {2}".format(
                        full_name, installed_version, version
                    )
                # If installed version is newer than new version
                else:
                    ret["comment"] = "{0} {1} (newer) is already installed".format(
                        name, installed_version
                    )
        # Catch all for a condition where version is not passed and there is no
        # available version
        else:
            ret["comment"] = "No version found to install"

    # Return if there are no changes to be made
    if not ret["changes"]:
        return ret

    # Return if running in test mode
    if __opts__["test"]:
        ret["result"] = None
        return ret

    # Install the package
    result = __salt__["chocolatey.upgrade"](
        name=name,
        version=version,
        source=source,
        force=force,
        pre_versions=pre_versions,
        install_args=install_args,
        override_args=override_args,
        force_x86=force_x86,
        package_args=package_args,
    )

    if "Running chocolatey failed" not in result:
        ret["comment"] = "Package {0} upgraded successfully".format(name)
        ret["result"] = True
    else:
        ret["comment"] = "Failed to upgrade the package {0}".format(name)
        ret["result"] = False

    # Get list of installed packages after 'chocolatey.install'
    post_install = __salt__["chocolatey.list"](local_only=True)

    # Prior to this, ret['changes'] would have contained expected changes,
    # replace them with the actual changes now that we have completed the
    # installation.
    ret["changes"] = salt.utils.data.compare_dicts(pre_install, post_install)

    return ret



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, unicode_literals

import logging
import os
import subprocess
import sys
import tempfile

# Import Salt libs
import salt
import salt.ext.six
import salt.modules.cmdmod
import salt.utils.files
import salt.utils.platform
import tests.support.helpers
from tests.support.runtests import RUNTIME_VARS

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf

log = logging.getLogger(__name__)


@skipIf(not salt.utils.path.which("bash"), "Bash needed for this test")
class VendorTornadoTest(TestCase):
    """
    Ensure we are not using any non vendor'ed tornado
    """

    def test_import_override(self):
        tmp = tempfile.mkdtemp()
        test_source = tests.support.helpers.dedent(
            """
        from __future__ import absolute_import, print_function
        import salt
        import tornado
        print(tornado.__name__)
        """
        )
        test_source_path = os.path.join(tmp, "test.py")
        tornado_source = tests.support.helpers.dedent(
            """
        foo = 'bar'
        """
        )
        tornado_source_path = os.path.join(tmp, "tornado.py")
        with salt.utils.files.fopen(test_source_path, "w") as fp:
            fp.write(test_source)
        with salt.utils.files.fopen(tornado_source_path, "w") as fp:
            fp.write(tornado_source)
        # Preserve the virtual environment
        env = os.environ.copy()
        if salt.utils.platform.is_windows():
            if salt.ext.six.PY2:
                env[b"PYTHONPATH"] = b";".join([a.encode() for a in sys.path])
            else:
                env["PYTHONPATH"] = ";".join(sys.path)
        else:
            env["PYTHONPATH"] = ":".join(sys.path)
        p = subprocess.Popen(
            [sys.executable, test_source_path],
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            env=env,
        )
        p.wait()
        pout = p.stdout.read().strip().decode()
        assert pout == "salt.ext.tornado", pout

    def test_vendored_tornado_import(self):
        grep_call = salt.modules.cmdmod.run_stdout(
            cmd="bash -c 'grep -r \"import tornado\" ./salt/*'",
            cwd=RUNTIME_VARS.CODE_DIR,
            ignore_retcode=True,
        ).split("\n")
        valid_lines = []
        for line in grep_call:
            if line == "":
                continue
            # Skip salt/ext/tornado/.. since there are a bunch of imports like
            # this in docstrings.
            if "salt/ext/tornado/" in line:
                continue
            log.error("Test found bad line: %s", line)
            valid_lines.append(line)
        assert valid_lines == [], len(valid_lines)

    def test_vendored_tornado_import_from(self):
        grep_call = salt.modules.cmdmod.run_stdout(
            cmd="bash -c 'grep -r \"from tornado\" ./salt/*'",
            cwd=RUNTIME_VARS.CODE_DIR,
            ignore_retcode=True,
        ).split("\n")
        valid_lines = []
        for line in grep_call:
            if line == "":
                continue
            log.error("Test found bad line: %s", line)
            valid_lines.append(line)
        assert valid_lines == [], len(valid_lines)

    def test_regression_56063(self):
        importer = salt.TornadoImporter()
        try:
            importer.find_module("tornado")
        except TypeError:
            assert False, "TornadoImporter raised type error when one argument passed"



# -*- coding: utf-8 -*-
"""
Unit tests for the boto_cloudfront state module.
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import copy
import textwrap

# Import Salt Libs
import salt.config
import salt.loader
import salt.states.boto_cloudfront as boto_cloudfront

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoCloudfrontTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_cloudfront
    """

    def setup_loader_modules(self):
        utils = salt.loader.utils(
            self.opts,
            whitelist=[
                "boto3",
                "dictdiffer",
                "yaml",
                "args",
                "systemd",
                "path",
                "platform",
            ],
            context={},
        )
        return {boto_cloudfront: {"__utils__": utils}}

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()

        cls.name = "my_distribution"
        cls.base_ret = {"name": cls.name, "changes": {}}

        # Most attributes elided since there are so many required ones
        cls.config = {"Enabled": True, "HttpVersion": "http2"}
        cls.tags = {"test_tag1": "value1"}

    @classmethod
    def tearDownClass(cls):
        del cls.opts

        del cls.name
        del cls.base_ret

        del cls.config
        del cls.tags

    def base_ret_with(self, extra_ret):
        new_ret = copy.deepcopy(self.base_ret)
        new_ret.update(extra_ret)
        return new_ret

    def test_present_distribution_retrieval_error(self):
        """
        Test for boto_cloudfront.present when we cannot get the distribution.
        """
        mock_get = MagicMock(return_value={"error": "get_distribution error"})
        with patch.multiple(
            boto_cloudfront,
            __salt__={"boto_cloudfront.get_distribution": mock_get},
            __opts__={"test": False},
        ):
            comment = "Error checking distribution {0}: get_distribution error"
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {"result": False, "comment": comment.format(self.name)}
                ),
            )

    def test_present_from_scratch(self):
        mock_get = MagicMock(return_value={"result": None})

        with patch.multiple(
            boto_cloudfront,
            __salt__={"boto_cloudfront.get_distribution": mock_get},
            __opts__={"test": True},
        ):
            comment = "Distribution {0} set for creation.".format(self.name)
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {
                        "result": None,
                        "comment": comment,
                        "changes": {"old": None, "new": self.name},
                    }
                ),
            )

        mock_create_failure = MagicMock(return_value={"error": "create error"})
        with patch.multiple(
            boto_cloudfront,
            __salt__={
                "boto_cloudfront.get_distribution": mock_get,
                "boto_cloudfront.create_distribution": mock_create_failure,
            },
            __opts__={"test": False},
        ):
            comment = "Error creating distribution {0}: create error"
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {"result": False, "comment": comment.format(self.name)}
                ),
            )

        mock_create_success = MagicMock(return_value={"result": True})
        with patch.multiple(
            boto_cloudfront,
            __salt__={
                "boto_cloudfront.get_distribution": mock_get,
                "boto_cloudfront.create_distribution": mock_create_success,
            },
            __opts__={"test": False},
        ):
            comment = "Created distribution {0}."
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {
                        "result": True,
                        "comment": comment.format(self.name),
                        "changes": {"old": None, "new": self.name},
                    }
                ),
            )

    def test_present_correct_state(self):
        mock_get = MagicMock(
            return_value={
                "result": {
                    "distribution": {"DistributionConfig": self.config},
                    "tags": self.tags,
                    "etag": "test etag",
                }
            }
        )
        with patch.multiple(
            boto_cloudfront,
            __salt__={"boto_cloudfront.get_distribution": mock_get},
            __opts__={"test": False},
        ):
            comment = "Distribution {0} has correct config."
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {"result": True, "comment": comment.format(self.name)}
                ),
            )

    def test_present_update_config_and_tags(self):
        mock_get = MagicMock(
            return_value={
                "result": {
                    "distribution": {
                        "DistributionConfig": {
                            "Enabled": False,
                            "Comment": "to be removed",
                        }
                    },
                    "tags": {"bad existing tag": "also to be removed"},
                    "etag": "test etag",
                }
            }
        )

        diff = textwrap.dedent(
            """\
            ---
            +++
            @@ -1,5 +1,5 @@
             config:
            -  Comment: to be removed
            -  Enabled: false
            +  Enabled: true
            +  HttpVersion: http2
             tags:
            -  bad existing tag: also to be removed
            +  test_tag1: value1

        """
        ).splitlines()
        # Difflib adds a trailing space after the +++/--- lines,
        # programatically add them back here. Having them in the test file
        # itself is not feasible since a few popular plugins for vim will
        # remove trailing whitespace.
        for idx in (0, 1):
            diff[idx] += " "
        diff = "\n".join(diff)

        with patch.multiple(
            boto_cloudfront,
            __salt__={"boto_cloudfront.get_distribution": mock_get},
            __opts__={"test": True},
        ):
            header = "Distribution {0} set for new config:".format(self.name)
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {
                        "result": None,
                        "comment": "\n".join([header, diff]),
                        "changes": {"diff": diff},
                    }
                ),
            )

        mock_update_failure = MagicMock(return_value={"error": "update error"})
        with patch.multiple(
            boto_cloudfront,
            __salt__={
                "boto_cloudfront.get_distribution": mock_get,
                "boto_cloudfront.update_distribution": mock_update_failure,
            },
            __opts__={"test": False},
        ):
            comment = "Error updating distribution {0}: update error"
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {"result": False, "comment": comment.format(self.name)}
                ),
            )

        mock_update_success = MagicMock(return_value={"result": True})
        with patch.multiple(
            boto_cloudfront,
            __salt__={
                "boto_cloudfront.get_distribution": mock_get,
                "boto_cloudfront.update_distribution": mock_update_success,
            },
            __opts__={"test": False},
        ):
            self.assertDictEqual(
                boto_cloudfront.present(self.name, self.config, self.tags),
                self.base_ret_with(
                    {
                        "result": True,
                        "comment": "Updated distribution {0}.".format(self.name),
                        "changes": {"diff": diff},
                    }
                ),
            )



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

import salt.utils.files
import salt.utils.platform
from tests.support.unit import skipIf


class JinjaFiltersTest(object):
    """
    testing Jinja filters are available via state system
    """

    def test_data_compare_dicts(self):
        """
        test jinja filter data.compare_dicts
        """
        _expected = {"ret": {"a": {"new": "c", "old": "b"}}}

        ret = self.run_function("state.sls", ["jinja_filters.data_compare_dicts"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_compare_lists(self):
        """
        test jinja filter data.compare_list
        """
        _expected = {"ret": {"old": ["b"]}}
        ret = self.run_function("state.sls", ["jinja_filters.data_compare_lists"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_decode_dict(self):
        """
        test jinja filter data.decode_dict
        """
        _expected = {"ret": {"a": "b", "c": "d"}}
        ret = self.run_function("state.sls", ["jinja_filters.data_decode_dict"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_data_decode_list(self):
        """
        test jinja filter data.decode_list
        """
        _expected = {"ret": ["a", "b", "c", "d"]}
        ret = self.run_function("state.sls", ["jinja_filters.data_decode_list"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_data_encode_dict(self):
        """
        test jinja filter data.encode_dict
        """
        _expected = {"ret": {"a": "b", "c": "d"}}
        ret = self.run_function("state.sls", ["jinja_filters.data_encode_dict"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_data_encode_list(self):
        """
        test jinja filter data.encode_list
        """
        _expected = {"ret": ["a", "b", "c", "d"]}
        ret = self.run_function("state.sls", ["jinja_filters.data_encode_list"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_data_exactly_n(self):
        """
        test jinja filter data.exactly_n
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.data_exactly_n"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_exactly_one(self):
        """
        test jinja filter data.exactly_one
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.data_exactly_one"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_is_iter(self):
        """
        test jinja filter data.is_iter
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.data_is_iter"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_is_list(self):
        """
        test jinja filter data.is_list
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.data_is_list"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_mysql_to_dict(self):
        """
        test jinja filter data.mysql_to_dict
        """
        _expected = {
            "ret": {
                "show processlist": {
                    "Info": "show processlist",
                    "db": "NULL",
                    "Host": "localhost",
                    "State": "init",
                    "Command": "Query",
                    "User": "root",
                    "Time": 0,
                    "Id": 7,
                }
            }
        }
        ret = self.run_function("state.sls", ["jinja_filters.data_mysql_to_dict"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_sorted_ignorecase(self):
        """
        test jinja filter data.softed_ignorecase
        """
        _expected = {"ret": ["Abcd", "efgh", "Ijk", "lmno", "Pqrs"]}
        ret = self.run_function("state.sls", ["jinja_filters.data_sorted_ignorecase"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_data_substring_in_list(self):
        """
        test jinja filter data.substring_in_list
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.data_substring_in_list"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_dateutils_strftime(self):
        """
        test jinja filter datautils.strftime
        """
        ret = self.run_function("state.sls", ["jinja_filters.dateutils_strftime"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_files_is_binary(self):
        """
        test jinja filter files.is_binary
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.files_is_binary"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_files_is_empty(self):
        """
        test jinja filter files.is_empty
        """
        try:
            if salt.utils.platform.is_windows():
                with salt.utils.files.fopen("c:\\empty_file", "w") as fp:
                    pass
            _expected = {"ret": True}
            ret = self.run_function("state.sls", ["jinja_filters.files_is_empty"])
            self.assertIn("module_|-test_|-test.echo_|-run", ret)
            self.assertEqual(
                ret["module_|-test_|-test.echo_|-run"]["changes"], _expected
            )
        finally:
            if salt.utils.platform.is_windows():
                os.remove("c:\\empty_file")

    def test_files_is_text(self):
        """
        test jinja filter files.is_text
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.files_is_text"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_files_list_files(self):
        """
        test jinja filter files.list_files
        """
        ret = self.run_function("state.sls", ["jinja_filters.files_list_files"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        if salt.utils.platform.is_windows():
            self.assertIn(
                "c:\\\\salt\\\\conf\\\\minion",
                ret["module_|-test_|-test.echo_|-run"]["changes"]["ret"],
            )
        else:
            self.assertIn(
                "/bin/ls", ret["module_|-test_|-test.echo_|-run"]["changes"]["ret"]
            )

    def test_hashutils_base4_64decode(self):
        """
        test jinja filter hashutils.base64_64decode
        """
        _expected = {"ret": "Salt Rocks!"}
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_base4_64decode"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_hashutils_base4_64encode(self):
        """
        test jinja filter hashutils.base64_64encode
        """
        _expected = {"ret": "U2FsdCBSb2NrcyE="}
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_base4_64encode"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_hashutils_file_hashsum(self):
        """
        test jinja filter hashutils.file_hashsum
        """
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_file_hashsum"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_hashutils_hmac(self):
        """
        test jinja filter hashutils.hmac
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_hmac"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_hashutils_md5_digest(self):
        """
        test jinja filter hashutils.md5_digest
        """
        _expected = {"ret": "85d6e71db655ee8e42c8b18475f0925f"}
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_md5_digest"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_hashutils_random_hash(self):
        """
        test jinja filter hashutils.random_hash
        """
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_random_hash"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_hashutils_sha256_digest(self):
        """
        test jinja filter hashutils.sha256_digest
        """
        _expected = {
            "ret": "cce7fe00fd9cc6122fd3b2ed22feae215bcfe7ac4a7879d336c1993426a763fe"
        }
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_sha256_digest"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_hashutils_sha512_digest(self):
        """
        test jinja filter hashutils.sha512_digest
        """
        _expected = {
            "ret": "44d829491d8caa7039ad08a5b7fa9dd0f930138c614411c5326dd4755fdc9877ec6148219fccbe404139e7bb850e77237429d64f560c204f3697ab489fd8bfa5"
        }
        ret = self.run_function("state.sls", ["jinja_filters.hashutils_sha512_digest"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_http_query(self):
        """
        test jinja filter http.query
        """
        _expected = {"ret": {}}
        ret = self.run_function("state.sls", ["jinja_filters.http_query"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_avg(self):
        """
        test jinja filter jinja.avg
        """
        ret = self.run_function("state.sls", ["jinja_filters.jinja_avg"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_jinja_difference(self):
        """
        test jinja filter jinja.difference
        """
        _expected = {"ret": [1, 3]}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_difference"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_intersect(self):
        """
        test jinja filter jinja.intersect
        """
        _expected = {"ret": [2, 4]}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_intersect"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_max(self):
        """
        test jinja filter jinja.max
        """
        _expected = {"ret": 4}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_max"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_min(self):
        """
        test jinja filter jinja.min
        """
        _expected = {"ret": 1}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_min"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_quote(self):
        """
        test jinja filter jinja.quote
        """
        _expected = {"ret": "Salt Rocks!"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_quote"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_regex_escape(self):
        """
        test jinja filter jinja.regex_escape
        """
        _expected = {"ret": "Salt\\ Rocks"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_regex_escape"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_regex_match(self):
        """
        test jinja filter jinja.regex_match
        """
        _expected = {"ret": "('a', 'd')"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_regex_match"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_regex_replace(self):
        """
        test jinja filter jinja.regex_replace
        """
        _expected = {"ret": "lets__replace__spaces"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_regex_replace"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_regex_search(self):
        """
        test jinja filter jinja.regex_search
        """
        _expected = {"ret": "('a', 'd')"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_regex_search"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_sequence(self):
        """
        test jinja filter jinja.sequence
        """
        _expected = {"ret": ["Salt Rocks!"]}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_sequence"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_skip(self):
        """
        test jinja filter jinja.skip
        """
        _expected = {"ret": None}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_skip"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_symmetric_difference(self):
        """
        test jinja filter jinja.symmetric_difference
        """
        _expected = {"ret": [1, 3, 6]}
        ret = self.run_function(
            "state.sls", ["jinja_filters.jinja_symmetric_difference"]
        )
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_to_bool(self):
        """
        test jinja filter jinja.to_bool
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_to_bool"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_union(self):
        """
        test jinja filter jinja.union
        """
        _expected = {"ret": [1, 2, 3, 4, 6]}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_union"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_unique(self):
        """
        test jinja filter jinja.unique
        """
        _expected = {"ret": ["a", "b", "c"]}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_unique"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_jinja_uuid(self):
        """
        test jinja filter jinja.uuid
        """
        _expected = {"ret": "799192d9-7f32-5227-a45f-dfeb4a34e06f"}
        ret = self.run_function("state.sls", ["jinja_filters.jinja_uuid"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_gen_mac(self):
        """
        test jinja filter network.gen_mac
        """
        _expected = "AC:DE:48:"
        ret = self.run_function("state.sls", ["jinja_filters.network_gen_mac"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertTrue(
            ret["module_|-test_|-test.echo_|-run"]["changes"]["ret"].startswith(
                _expected
            )
        )

    def test_network_ipaddr(self):
        """
        test jinja filter network.ipaddr
        """
        ret = self.run_function("state.sls", ["jinja_filters.network_ipaddr"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_network_ip_host(self):
        """
        test jinja filter network.ip_host
        """
        _expected = {"ret": "192.168.0.12/24"}
        ret = self.run_function("state.sls", ["jinja_filters.network_ip_host"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_ipv4(self):
        """
        test jinja filter network.ipv4
        """
        _expected = {"ret": ["127.0.0.1"]}
        ret = self.run_function("state.sls", ["jinja_filters.network_ipv4"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_ipv6(self):
        """
        test jinja filter network.ipv6
        """
        ret = self.run_function("state.sls", ["jinja_filters.network_ipv6"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_network_is_ip(self):
        """
        test jinja filter network.is_ip
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.network_is_ip"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_is_ipv4(self):
        """
        test jinja filter network.is_ipv4
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.network_is_ipv4"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_is_ipv6(self):
        """
        test jinja filter network.is_ipv6
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.network_is_ipv6"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_network_hosts(self):
        """
        test jinja filter network.network_hosts
        """
        _expected = {"ret": ["192.168.1.1", "192.168.1.2"]}
        ret = self.run_function("state.sls", ["jinja_filters.network_network_hosts"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_network_network_size(self):
        """
        test jinja filter network.network_size
        """
        _expected = {"ret": 16}
        ret = self.run_function("state.sls", ["jinja_filters.network_network_size"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_path_join(self):
        """
        test jinja filter path.join
        """
        _expected = {"ret": os.path.sep + os.path.join("a", "b", "c", "d")}
        ret = self.run_function("state.sls", ["jinja_filters.path_join"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_path_which(self):
        """
        test jinja filter path.which
        """
        ret = self.run_function("state.sls", ["jinja_filters.path_which"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_stringutils_contains_whitespace(self):
        """
        test jinja filter stringutils.contains_whitespace
        """
        _expected = {"ret": True}
        ret = self.run_function(
            "state.sls", ["jinja_filters.stringutils_contains_whitespace"]
        )
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_stringutils_is_hex(self):
        """
        test jinja filter stringutils.is_hex
        """
        _expected = {"ret": True}
        ret = self.run_function("state.sls", ["jinja_filters.stringutils_is_hex"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_stringutils_random_str(self):
        """
        test jinja filter stringutils.random_str
        """
        ret = self.run_function("state.sls", ["jinja_filters.stringutils_random_str"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_stringutils_to_bytes(self):
        """
        test jinja filter stringutils.to_bytes
        """
        _expected = {"ret": "Salt Rocks!"}
        ret = self.run_function("state.sls", ["jinja_filters.stringutils_to_bytes"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])

    def test_stringutils_to_num(self):
        """
        test jinja filter stringutils.to_num
        """
        _expected = {"ret": 42}
        ret = self.run_function("state.sls", ["jinja_filters.stringutils_to_num"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_stringutils_whitelist_blacklist(self):
        """
        test jinja filter stringutils.whitelist_blacklist
        """
        _expected = {"ret": True}
        ret = self.run_function(
            "state.sls", ["jinja_filters.stringutils_whitelist_blacklist"]
        )
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    @skipIf(salt.utils.platform.is_windows(), "Skip on windows")
    def test_user_get_uid(self):
        """
        test jinja filter user.get_uid
        """
        _expected = {"ret": 0}
        ret = self.run_function("state.sls", ["jinja_filters.user_get_uid"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_yamlencoding_yaml_dquote(self):
        """
        test jinja filter yamlencoding.yaml_dquote
        """
        _expected = {"ret": "A double-quoted string in YAML"}
        ret = self.run_function("state.sls", ["jinja_filters.yamlencoding_yaml_dquote"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_yamlencoding_yaml_encode(self):
        """
        test jinja filter yamlencoding.yaml_encode
        """
        _expected = {"ret": "An encoded string in YAML"}
        ret = self.run_function("state.sls", ["jinja_filters.yamlencoding_yaml_encode"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_yamlencoding_yaml_squote(self):
        """
        test jinja filter yamlencoding.yaml_squote
        """
        _expected = {"ret": "A single-quoted string in YAML"}
        ret = self.run_function("state.sls", ["jinja_filters.yamlencoding_yaml_squote"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_yaml(self):
        """
        test yaml filter
        """
        _expected = {"ret": "{Question: 'Quieres Café?'}"}
        ret = self.run_function("state.sls", ["jinja_filters.yaml"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)

    def test_json(self):
        """
        test json filter
        """
        _expected = {"ret": '{"Question": "Quieres Café?"}'}
        ret = self.run_function("state.sls", ["jinja_filters.json"])
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertEqual(ret["module_|-test_|-test.echo_|-run"]["changes"], _expected)



# coding: utf-8

# Python libs
from __future__ import absolute_import

import logging

# Salt libs
import salt.beacons.load as load
import salt.utils.platform
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase, skipIf

log = logging.getLogger(__name__)


class LoadBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.load
    """

    def setup_loader_modules(self):
        return {load: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = load.validate(config)

        self.assertEqual(ret, (False, "Configuration for load beacon must be a list."))

    def test_empty_config(self):
        config = [{}]

        ret = load.validate(config)

        self.assertEqual(
            ret, (False, "Averages configuration is required for load beacon.")
        )

    @skipIf(salt.utils.platform.is_windows(), "os.getloadavg not available on Windows")
    def test_load_match(self):
        with patch("os.getloadavg", MagicMock(return_value=(1.82, 1.84, 1.56))):
            config = [
                {
                    "averages": {"1m": [0.0, 2.0], "5m": [0.0, 1.5], "15m": [0.0, 1.0]},
                    "emitatstartup": True,
                    "onchangeonly": False,
                }
            ]

            ret = load.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            _expected_return = [{"1m": 1.82, "5m": 1.84, "15m": 1.56}]
            ret = load.beacon(config)
            self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
    :codeauthor: Joe Julian <me@joejulian.name>
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.glusterfs as glusterfs
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class GlusterResults(object):
    """ This class holds the xml results from gluster cli transactions """

    class v34(object):
        """ This is for version 3.4 results """

        class list_peers(object):
            """ results from "peer status" """

        class peer_probe(object):
            fail_cant_connect = fail_bad_hostname = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>',
                    "<cliOutput>",
                    "  <opRet>-1</opRet>",
                    "  <opErrno>107</opErrno>",
                    "  <opErrstr>Probe returned with unknown errno 107</opErrstr>",
                    "</cliOutput>",
                    "",
                ]
            )

            success_self = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                    "  <cliOutput>",
                    "  <opRet>0</opRet>",
                    "  <opErrno>1</opErrno>",
                    "  <opErrstr>(null)</opErrstr>",
                    "  <output>success: on localhost not needed</output>",
                    "</cliOutput>",
                    "",
                ]
            )
            success_other = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                    "  <cliOutput>",
                    "  <opRet>0</opRet>",
                    "  <opErrno>0</opErrno>",
                    "  <opErrstr>(null)</opErrstr>",
                    "  <output>success</output>",
                    "</cliOutput>",
                    "",
                ]
            )
            success_hostname_after_ip = success_other
            success_ip_after_hostname = success_other
            success_already_peer = {
                "ip": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr>(null)</opErrstr>",
                        "  <output>success: host 10.0.0.2 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
                "hostname": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr>(null)</opErrstr>",
                        "  <output>success: host server2 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
            }
            success_reverse_already_peer = {
                "ip": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr>(null)</opErrstr>",
                        "  <output>success: host 10.0.0.1 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
                "hostname": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr>(null)</opErrstr>",
                        "  <output>success: host server1 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
            }
            success_first_hostname_from_second_first_time = success_other
            success_first_hostname_from_second_second_time = success_reverse_already_peer[
                "hostname"
            ]
            success_first_ip_from_second_first_time = success_reverse_already_peer["ip"]

    class v37(object):
        class peer_probe(object):
            fail_cant_connect = fail_bad_hostname = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>',
                    "<cliOutput>",
                    "  <opRet>-1</opRet>",
                    "  <opErrno>107</opErrno>",
                    "  <opErrstr>Probe returned with Transport endpoint is not connected</opErrstr>",
                    "</cliOutput>",
                    "",
                ]
            )
            success_self = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                    "  <cliOutput>",
                    "  <opRet>0</opRet>",
                    "  <opErrno>1</opErrno>",
                    "  <opErrstr/>",
                    "  <output>Probe on localhost not needed</output>",
                    "</cliOutput>",
                    "",
                ]
            )
            success_other = "\n".join(
                [
                    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                    "  <cliOutput>",
                    "  <opRet>0</opRet>",
                    "  <opErrno>0</opErrno>",
                    "  <opErrstr/>",
                    "  <output/>",
                    "</cliOutput>",
                    "",
                ]
            )
            success_hostname_after_ip = success_other
            success_ip_after_hostname = success_other
            success_already_peer = {
                "ip": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr/>",
                        "  <output>Host 10.0.0.2 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
                "hostname": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr/>",
                        "  <output>Host server2 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
            }
            success_reverse_already_peer = {
                "ip": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr/>",
                        "  <output>Host 10.0.0.1 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
                "hostname": "\n".join(
                    [
                        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
                        "  <cliOutput>",
                        "  <opRet>0</opRet>",
                        "  <opErrno>2</opErrno>",
                        "  <opErrstr/>",
                        "  <output>Host server1 port 24007 already in peer list</output>",
                        "</cliOutput>",
                        "",
                    ]
                ),
            }
            success_first_hostname_from_second_first_time = success_reverse_already_peer[
                "hostname"
            ]
            success_first_ip_from_second_first_time = success_other
            success_first_ip_from_second_second_time = success_reverse_already_peer[
                "ip"
            ]


xml_peer_present = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <peer>
    <uuid>uuid1</uuid>
    <hostname>node02</hostname>
    <hostnames>
      <hostname>node02.domain.dom</hostname>
      <hostname>10.0.0.2</hostname>
    </hostnames>
  </peer>
</cliOutput>
"""

xml_volume_present = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <volList>
    <volume>Newvolume1</volume>
    <volume>Newvolume2</volume>
  </volList>
</cliOutput>
"""

xml_volume_absent = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <volList>
    <count>0</count>
  </volList>
</cliOutput>
"""

xml_volume_status = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <volStatus>
    <volumes>
      <volume>
        <volName>myvol1</volName>
        <nodeCount>3</nodeCount>
        <node>
          <hostname>node01</hostname>
          <path>/tmp/foo</path>
          <peerid>830700d7-0684-497c-a12c-c02e365fb90b</peerid>
          <status>1</status>
          <port>49155</port>
          <ports>
            <tcp>49155</tcp>
            <rdma>N/A</rdma>
          </ports>
          <pid>2470</pid>
        </node>
        <node>
          <hostname>NFS Server</hostname>
          <path>localhost</path>
          <peerid>830700d7-0684-497c-a12c-c02e365fb90b</peerid>
          <status>0</status>
          <port>N/A</port>
          <ports>
            <tcp>N/A</tcp>
            <rdma>N/A</rdma>
          </ports>
          <pid>-1</pid>
        </node>
        <tasks/>
      </volume>
    </volumes>
  </volStatus>
</cliOutput>
"""

xml_volume_info_running = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <volInfo>
    <volumes>
      <volume>
        <name>myvol1</name>
        <id>f03c2180-cf55-4f77-ae0b-3650f57c82a1</id>
        <status>1</status>
        <statusStr>Started</statusStr>
        <brickCount>1</brickCount>
        <distCount>1</distCount>
        <stripeCount>1</stripeCount>
        <replicaCount>1</replicaCount>
        <disperseCount>0</disperseCount>
        <redundancyCount>0</redundancyCount>
        <type>0</type>
        <typeStr>Distribute</typeStr>
        <transport>0</transport>
        <bricks>
          <brick uuid="830700d7-0684-497c-a12c-c02e365fb90b">node01:/tmp/foo<name>node01:/tmp/foo</name><hostUuid>830700d7-0684-497c-a12c-c02e365fb90b</hostUuid></brick>
        </bricks>
        <optCount>1</optCount>
        <options>
          <option>
            <name>performance.readdir-ahead</name>
            <value>on</value>
          </option>
        </options>
      </volume>
      <count>1</count>
    </volumes>
  </volInfo>
</cliOutput>
"""

xml_volume_info_stopped = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <volInfo>
    <volumes>
      <volume>
        <name>myvol1</name>
        <status>1</status>
      </volume>
    </volumes>
  </volInfo>
</cliOutput>
"""

xml_peer_probe_success = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>0</opErrno>
  <opErrstr/>
  <output/>
</cliOutput>
"""

xml_peer_probe_already_member = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>2</opErrno>
  <opErrstr/>
  <output>Host salt port 24007 already in peer list</output>
</cliOutput>
"""

xml_peer_probe_localhost = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>1</opErrno>
  <opErrstr/>
  <output>Probe on localhost not needed</output>
</cliOutput>
"""

xml_peer_probe_fail_cant_connect = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>-1</opRet>
  <opErrno>107</opErrno>
  <opErrstr>Probe returned with Transport endpoint is not connected</opErrstr>
</cliOutput>
"""

xml_command_success = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
</cliOutput>
"""

xml_command_fail = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>-1</opRet>
    <opErrno>0</opErrno>
  <opErrstr>Command Failed</opErrstr>
</cliOutput>
"""

xml_op_version_37 = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>0</opErrno>
  <opErrstr/>
  <volGetopts>
    <count>1</count>
    <Option>cluster.op-version</Option>
    <Value>30707</Value>
  </volGetopts>
</cliOutput>
"""

xml_op_version_312 = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>0</opErrno>
  <opErrstr/>
  <volGetopts>
    <count>1</count>
    <Opt>
      <Option>cluster.op-version</Option>
      <Value>30707</Value>
    </Opt>
  </volGetopts>
</cliOutput>
"""

xml_max_op_version = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>0</opErrno>
  <opErrstr/>
  <volGetopts>
    <count>1</count>
    <Opt>
      <Option>cluster.max-op-version</Option>
      <Value>31200</Value>
    </Opt>
  </volGetopts>
</cliOutput>
"""

xml_set_op_version_failure = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>-1</opRet>
  <opErrno>30800</opErrno>
  <opErrstr>Required op-version (30707) should not be equal or lower than current cluster op-version (30707).</opErrstr>
  <cliOp>volSet</cliOp>
  <output>Set volume unsuccessful</output>
</cliOutput>
"""

xml_set_op_version_success = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<cliOutput>
  <opRet>0</opRet>
  <opErrno>0</opErrno>
  <opErrstr/>
  <cliOp>volSet</cliOp>
  <output>Set volume successful</output>
</cliOutput>
"""


class GlusterfsTestCase(TestCase, LoaderModuleMockMixin):

    """
    Test cases for salt.modules.glusterfs
    """

    def setup_loader_modules(self):
        return {glusterfs: {}}

    maxDiff = None

    # 'peer_status' function tests: 1

    def test_peer_status(self):
        """
        Test gluster peer status
        """
        mock_run = MagicMock(return_value=xml_peer_present)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertDictEqual(
                glusterfs.peer_status(),
                {"uuid1": {"hostnames": ["node02", "node02.domain.dom", "10.0.0.2"]}},
            )

        mock_run = MagicMock(return_value=xml_command_success)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertDictEqual(glusterfs.peer_status(), {})

    # 'peer' function tests: 1

    def test_peer(self):
        """
        Test if gluster peer call is successful.
        """
        mock_run = MagicMock()
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            mock_run.return_value = xml_peer_probe_already_member
            self.assertTrue(glusterfs.peer("salt"))

            mock_run.return_value = xml_peer_probe_localhost
            self.assertTrue(glusterfs.peer("salt"))

            mock_run.return_value = xml_peer_probe_fail_cant_connect
            self.assertFalse(glusterfs.peer("salt"))

    # 'create_volume' function tests: 1

    def test_create_volume(self):
        """
        Test if it creates a glusterfs volume.
        """
        mock_run = MagicMock(return_value=xml_command_success)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertRaises(
                SaltInvocationError, glusterfs.create_volume, "newvolume", "host1:brick"
            )

            self.assertRaises(
                SaltInvocationError, glusterfs.create_volume, "newvolume", "host1/brick"
            )

            self.assertFalse(mock_run.called)

            mock_start_volume = MagicMock(return_value=True)
            with patch.object(glusterfs, "start_volume", mock_start_volume):
                # Create, do not start
                self.assertTrue(glusterfs.create_volume("newvolume", "host1:/brick"))
                self.assertFalse(mock_start_volume.called)

                # Create and start
                self.assertTrue(
                    glusterfs.create_volume("newvolume", "host1:/brick", start=True)
                )
                self.assertTrue(mock_start_volume.called)

                mock_start_volume.return_value = False
                # Create and fail start
                self.assertFalse(
                    glusterfs.create_volume("newvolume", "host1:/brick", start=True)
                )

            mock_run.return_value = xml_command_fail
            self.assertFalse(
                glusterfs.create_volume(
                    "newvolume", "host1:/brick", True, True, True, "tcp", True
                )
            )

    # 'list_volumes' function tests: 1

    def test_list_volumes(self):
        """
        Test if it list configured volumes
        """
        mock = MagicMock(return_value=xml_volume_absent)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock}):
            self.assertListEqual(glusterfs.list_volumes(), [])

        mock = MagicMock(return_value=xml_volume_present)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock}):
            self.assertListEqual(glusterfs.list_volumes(), ["Newvolume1", "Newvolume2"])

    # 'status' function tests: 1

    def test_status(self):
        """
        Test if it check the status of a gluster volume.
        """
        mock_run = MagicMock(return_value=xml_command_fail)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertIsNone(glusterfs.status("myvol1"))

        res = {
            "bricks": {
                "node01:/tmp/foo": {
                    "host": "node01",
                    "hostname": "node01",
                    "online": True,
                    "path": "/tmp/foo",
                    "peerid": "830700d7-0684-497c-a12c-c02e365fb90b",
                    "pid": "2470",
                    "port": "49155",
                    "ports": {"rdma": "N/A", "tcp": "49155"},
                    "status": "1",
                }
            },
            "healers": {},
            "nfs": {
                "node01": {
                    "host": "NFS Server",
                    "hostname": "NFS Server",
                    "online": False,
                    "path": "localhost",
                    "peerid": "830700d7-0684-497c-a12c-c02e365fb90b",
                    "pid": "-1",
                    "port": "N/A",
                    "ports": {"rdma": "N/A", "tcp": "N/A"},
                    "status": "0",
                }
            },
        }
        mock = MagicMock(return_value=xml_volume_status)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(glusterfs.status("myvol1"), res)

    # 'start_volume' function tests: 1

    def test_volume_info(self):
        """
        Test if it returns the volume info.
        """
        res = {
            "myvol1": {
                "brickCount": "1",
                "bricks": {
                    "brick1": {
                        "hostUuid": "830700d7-0684-497c-a12c-c02e365fb90b",
                        "path": "node01:/tmp/foo",
                        "uuid": "830700d7-0684-497c-a12c-c02e365fb90b",
                    }
                },
                "disperseCount": "0",
                "distCount": "1",
                "id": "f03c2180-cf55-4f77-ae0b-3650f57c82a1",
                "name": "myvol1",
                "optCount": "1",
                "options": {"performance.readdir-ahead": "on"},
                "redundancyCount": "0",
                "replicaCount": "1",
                "status": "1",
                "statusStr": "Started",
                "stripeCount": "1",
                "transport": "0",
                "type": "0",
                "typeStr": "Distribute",
            }
        }
        mock = MagicMock(return_value=xml_volume_info_running)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(glusterfs.info("myvol1"), res)

    def test_start_volume(self):
        """
        Test if it start a gluster volume.
        """
        # Stopped volume
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "0"}})
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.start_volume("Newvolume1"), True)
                self.assertEqual(glusterfs.start_volume("nonExisting"), False)
            mock_run = MagicMock(return_value=xml_command_fail)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.start_volume("Newvolume1"), False)

        # Started volume
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "1"}})
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.start_volume("Newvolume1", force=True), True)
            mock_run = MagicMock(return_value=xml_command_fail)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                # cmd.run should not be called for already running volume:
                self.assertEqual(glusterfs.start_volume("Newvolume1"), True)
                # except when forcing:
                self.assertEqual(
                    glusterfs.start_volume("Newvolume1", force=True), False
                )

    # 'stop_volume' function tests: 1

    def test_stop_volume(self):
        """
        Test if it stop a gluster volume.
        """
        # Stopped volume
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "0"}})
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.stop_volume("Newvolume1"), True)
                self.assertEqual(glusterfs.stop_volume("nonExisting"), False)
            mock_run = MagicMock(return_value=xml_command_fail)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                # cmd.run should not be called for already stopped volume:
                self.assertEqual(glusterfs.stop_volume("Newvolume1"), True)

        # Started volume
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "1"}})
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.stop_volume("Newvolume1"), True)
                self.assertEqual(glusterfs.stop_volume("nonExisting"), False)
            mock_run = MagicMock(return_value=xml_command_fail)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertEqual(glusterfs.stop_volume("Newvolume1"), False)

    # 'delete_volume' function tests: 1

    def test_delete_volume(self):
        """
        Test if it deletes a gluster volume.
        """
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "1"}})
        with patch.object(glusterfs, "info", mock_info):
            # volume doesn't exist
            self.assertFalse(glusterfs.delete_volume("Newvolume3"))

            mock_stop_volume = MagicMock(return_value=True)
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                with patch.object(glusterfs, "stop_volume", mock_stop_volume):
                    # volume exists, should not be stopped, and is started
                    self.assertFalse(glusterfs.delete_volume("Newvolume1", False))
                    self.assertFalse(mock_run.called)
                    self.assertFalse(mock_stop_volume.called)

                    # volume exists, should be stopped, and is started
                    self.assertTrue(glusterfs.delete_volume("Newvolume1"))
                    self.assertTrue(mock_run.called)
                    self.assertTrue(mock_stop_volume.called)

        # volume exists and isn't started
        mock_info = MagicMock(return_value={"Newvolume1": {"status": "2"}})
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                self.assertTrue(glusterfs.delete_volume("Newvolume1"))
                mock_run.return_value = xml_command_fail
                self.assertFalse(glusterfs.delete_volume("Newvolume1"))

    # 'add_volume_bricks' function tests: 1

    def test_add_volume_bricks(self):
        """
        Test if it add brick(s) to an existing volume
        """
        mock_info = MagicMock(
            return_value={
                "Newvolume1": {
                    "status": "1",
                    "bricks": {
                        "brick1": {"path": "host:/path1"},
                        "brick2": {"path": "host:/path2"},
                    },
                }
            }
        )
        with patch.object(glusterfs, "info", mock_info):
            mock_run = MagicMock(return_value=xml_command_success)
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
                # Volume does not exist
                self.assertFalse(glusterfs.add_volume_bricks("nonExisting", ["bricks"]))
                # Brick already exists
                self.assertTrue(
                    glusterfs.add_volume_bricks("Newvolume1", ["host:/path2"])
                )
                # Already existing brick as a string
                self.assertTrue(
                    glusterfs.add_volume_bricks("Newvolume1", "host:/path2")
                )
                self.assertFalse(mock_run.called)
                # A new brick:
                self.assertTrue(
                    glusterfs.add_volume_bricks("Newvolume1", ["host:/new1"])
                )
                self.assertTrue(mock_run.called)

                # Gluster call fails
                mock_run.return_value = xml_command_fail
                self.assertFalse(
                    glusterfs.add_volume_bricks("Newvolume1", ["new:/path"])
                )

    # 'get_op_version' function tests: 1

    def test_get_op_version(self):
        """
        Test retrieving the glusterfs op-version
        """

        # Test with xml output structure from v3.7
        mock_run = MagicMock(return_value=xml_op_version_37)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertEqual(glusterfs.get_op_version("test"), "30707")

        # Test with xml output structure from v3.12
        mock_run = MagicMock(return_value=xml_op_version_312)
        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_run}):
            self.assertEqual(glusterfs.get_op_version("test"), "30707")

    # 'get_max_op_version' function tests: 1

    def test_get_max_op_version(self):
        """
        Test retrieving the glusterfs max-op-version.
        """

        mock_xml = MagicMock(return_value=xml_max_op_version)
        mock_version = MagicMock(return_value="glusterfs 3.9.1")

        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_version}):
            self.assertFalse(glusterfs.get_max_op_version()[0])

        with patch.object(glusterfs, "_get_version", return_value=(3, 12, 0)):
            with patch.dict(glusterfs.__salt__, {"cmd.run": mock_xml}):
                self.assertEqual(glusterfs.get_max_op_version(), "31200")

    # 'set_op_version' function tests: 1

    def test_set_op_version(self):
        """
        Test setting the glusterfs op-version
        """
        mock_failure = MagicMock(return_value=xml_set_op_version_failure)
        mock_success = MagicMock(return_value=xml_set_op_version_success)

        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_failure}):
            self.assertFalse(glusterfs.set_op_version(30707)[0])

        with patch.dict(glusterfs.__salt__, {"cmd.run": mock_success}):
            self.assertEqual(glusterfs.set_op_version(31200), "Set volume successful")



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.modules.mac_assistive as assistive

# Import Salt Libs
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AssistiveTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {assistive: {}}

    def test_install_assistive_bundle(self):
        """
        Test installing a bundle ID as being allowed to run with assistive access
        """
        mock_ret = MagicMock(return_value={"retcode": 0})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            with patch.dict(assistive.__grains__, {"osrelease": "10.11.3"}):
                self.assertTrue(assistive.install("foo"))

    def test_install_assistive_error(self):
        """
        Test installing a bundle ID as being allowed to run with assistive access
        """
        mock_ret = MagicMock(return_value={"retcode": 1})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            with patch.dict(assistive.__grains__, {"osrelease": "10.11.3"}):
                self.assertRaises(CommandExecutionError, assistive.install, "foo")

    def test_installed_bundle(self):
        """
        Test checking to see if a bundle id is installed as being able to use assistive access
        """
        with patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[("foo", 0)]),
        ):
            self.assertTrue(assistive.installed("foo"))

    def test_installed_bundle_not(self):
        """
        Test checking to see if a bundle id is installed as being able to use assistive access
        """
        with patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[]),
        ):
            self.assertFalse(assistive.installed("foo"))

    def test_enable_assistive(self):
        """
        Test enabling a bundle ID as being allowed to run with assistive access
        """
        mock_ret = MagicMock(return_value={"retcode": 0})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}), patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[("foo", 0)]),
        ):
            self.assertTrue(assistive.enable("foo", True))

    def test_enable_error(self):
        """
        Test enabled a bundle ID that throws a command error
        """
        mock_ret = MagicMock(return_value={"retcode": 1})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}), patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[("foo", 0)]),
        ):
            self.assertRaises(CommandExecutionError, assistive.enable, "foo")

    def test_enable_false(self):
        """
        Test return of enable function when app isn't found.
        """
        with patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[]),
        ):
            self.assertFalse(assistive.enable("foo"))

    def test_enabled_assistive(self):
        """
        Test enabling a bundle ID as being allowed to run with assistive access
        """
        with patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[("foo", "1")]),
        ):
            self.assertTrue(assistive.enabled("foo"))

    def test_enabled_assistive_false(self):
        """
        Test if a bundle ID is disabled for assistive access
        """
        with patch(
            "salt.modules.mac_assistive._get_assistive_access",
            MagicMock(return_value=[]),
        ):
            self.assertFalse(assistive.enabled("foo"))

    def test_remove_assistive(self):
        """
        Test removing an assitive bundle.
        """
        mock_ret = MagicMock(return_value={"retcode": 0})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            self.assertTrue(assistive.remove("foo"))

    def test_remove_assistive_error(self):
        """
        Test removing an assitive bundle.
        """
        mock_ret = MagicMock(return_value={"retcode": 1})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            self.assertRaises(CommandExecutionError, assistive.remove, "foo")

    def test_get_assistive_access(self):
        """
        Test if a bundle ID is enabled for assistive access
        """
        mock_out = (
            "kTCCServiceAccessibility|/bin/bash|1|1|1|\n"
            "kTCCServiceAccessibility|/usr/bin/osascript|1|1|1|"
        )
        mock_ret = MagicMock(return_value={"retcode": 0, "stdout": mock_out})
        expected = [("/bin/bash", "1"), ("/usr/bin/osascript", "1")]
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            self.assertEqual(assistive._get_assistive_access(), expected)

    def test_get_assistive_access_error(self):
        """
        Test a CommandExecutionError is raised when something goes wrong.
        """
        mock_ret = MagicMock(return_value={"retcode": 1})
        with patch.dict(assistive.__salt__, {"cmd.run_all": mock_ret}):
            self.assertRaises(CommandExecutionError, assistive._get_assistive_access)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.rabbitmq as rabbitmq
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class RabbitmqTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.rabbitmq
    """

    def setup_loader_modules(self):
        return {
            rabbitmq: {"__context__": {"rabbitmqctl": None, "rabbitmq-plugins": None}}
        }

    # 'list_users_rabbitmq2' function tests: 1

    def test_list_users_rabbitmq2(self):
        """
        Test if it return a list of users based off of rabbitmqctl user_list.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing users ...\nguest\t[administrator, user]\njustAnAdmin\t[administrator]\n",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_users(),
                {"guest": ["administrator", "user"], "justAnAdmin": ["administrator"]},
            )

    # 'list_users_rabbitmq3' function tests: 1

    def test_list_users_rabbitmq3(self):
        """
        Test if it return a list of users based off of rabbitmqctl user_list.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "guest\t[administrator user]\r\nother\t[a b]\r\n",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_users(),
                {"guest": ["administrator", "user"], "other": ["a", "b"]},
            )

    # 'list_users_with_warning_rabbitmq2' function tests: 1

    def test_list_users_with_warning_rabbitmq2(self):
        """
        Test if having a leading WARNING returns the user_list anyway.
        """
        rtn_stdout = "\n".join(
            [
                "WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf",
                "Listing users ...",
                "guest\t[administrator, user]\n",
            ]
        )
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": rtn_stdout, "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_users(), {"guest": ["administrator", "user"]}
            )

    # 'list_users_with_warning_rabbitmq3' function tests: 1

    def test_list_users_with_warning_rabbitmq3(self):
        """
        Test if having a leading WARNING returns the user_list anyway.
        """
        rtn_stdout = "\n".join(
            [
                "WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf",
                "Listing users ...",
                "guest\t[administrator user]\n",
            ]
        )
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": rtn_stdout, "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_users(), {"guest": ["administrator", "user"]}
            )

    # 'list_vhosts' function tests: 2

    def test_list_vhosts(self):
        """
        Test if it return a list of vhost based on rabbitmqctl list_vhosts.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "/\nsaltstack\n...", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertListEqual(rabbitmq.list_vhosts(), ["/", "saltstack", "..."])

    def test_list_vhosts_with_warning(self):
        """
        Test if it return a list of vhost based on rabbitmqctl list_vhosts even with a leading WARNING.
        """
        rtn_stdout = "\n".join(
            [
                "WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf",
                "Listing users ...",
                "/",
                "saltstack",
                "...\n",
            ]
        )
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": rtn_stdout, "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertListEqual(rabbitmq.list_vhosts(), ["/", "saltstack", "..."])

    # 'user_exists' function tests: 2

    def test_user_exists(self):
        """
        Test whether a given rabbitmq-internal user exists based
        on rabbitmqctl list_users.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing users ...\n" "saltstack\t[administrator]\n...done",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertTrue(rabbitmq.user_exists("saltstack"))

    def test_user_exists_negative(self):
        """
        Negative test of whether rabbitmq-internal user exists based
        on rabbitmqctl list_users.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing users ...\n" "saltstack\t[administrator]\n...done",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertFalse(rabbitmq.user_exists("salt"))

    # 'vhost_exists' function tests: 2

    def test_vhost_exists(self):
        """
        Test if it return whether the vhost exists based
        on rabbitmqctl list_vhosts.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing vhosts ...\nsaltstack",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertTrue(rabbitmq.vhost_exists("saltstack"))

    def test_vhost_exists_negative(self):
        """
        Test if it return whether the vhost exists based
        on rabbitmqctl list_vhosts.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing vhosts ...\nsaltstack",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertFalse(rabbitmq.vhost_exists("salt"))

    # 'add_user' function tests: 1

    def test_add_user(self):
        """
        Test if it add a rabbitMQ user via rabbitmqctl
        user_add <user> <password>
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(rabbitmq.add_user("saltstack"), {"Added": "saltstack"})

        mock_run = MagicMock(return_value="Error")
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            with patch.object(
                rabbitmq,
                "clear_password",
                return_value={"Error": "Error", "retcode": 1},
            ):
                self.assertRaises(CommandExecutionError, rabbitmq.add_user, "saltstack")

    # 'delete_user' function tests: 1

    def test_delete_user(self):
        """
        Test if it deletes a user via rabbitmqctl delete_user.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.delete_user("saltstack"), {"Deleted": "saltstack"}
            )

    # 'check_password' function tests: 2

    def test_check_password_lt_38(self):
        """
        Test if it checks a user's password for RabbitMQ less than v3.8.
        """
        mock_run = MagicMock(return_value='{rabbit,"RabbitMQ","3.5.7"}')
        mock_run2 = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": 'Authenticating user "saltstack" ...\nSuccess',
                "stderr": "",
            }
        )
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run": mock_run, "cmd.run_all": mock_run2}
        ):
            self.assertEqual(rabbitmq.check_password("saltstack", "salt@123"), True)

    def test_check_password_gt_38(self):
        """
        Test if it checks a user's password for RabbitMQ greater than v3.8.
        """
        mock_run = MagicMock(return_value="RabbitMQ version: 3.8.3")
        mock_run2 = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": 'Authenticating user "saltstack" ...\nSuccess',
                "stderr": "",
            }
        )
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run": mock_run, "cmd.run_all": mock_run2}
        ):
            self.assertEqual(rabbitmq.check_password("saltstack", "salt@123"), True)

    # 'change_password' function tests: 1

    def test_change_password(self):
        """
        Test if it changes a user's password.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.change_password("saltstack", "salt@123"),
                {"Password Changed": "saltstack"},
            )

    # 'clear_password' function tests: 1

    def test_clear_password(self):
        """
        Test if it removes a user's password.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.clear_password("saltstack"), {"Password Cleared": "saltstack"}
            )

    # 'add_vhost' function tests: 1

    def test_add_vhost(self):
        """
        Test if it adds a vhost via rabbitmqctl add_vhost.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.add_vhost("saltstack"), {"Added": "saltstack"}
            )

    # 'delete_vhost' function tests: 1

    def test_delete_vhost(self):
        """
        Test if it deletes a vhost rabbitmqctl delete_vhost.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.delete_vhost("saltstack"), {"Deleted": "saltstack"}
            )

    # 'set_permissions' function tests: 1

    def test_set_permissions(self):
        """
        Test if it sets permissions for vhost via rabbitmqctl set_permissions.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.set_permissions("myvhost", "myuser"),
                {"Permissions Set": "saltstack"},
            )

    # 'list_permissions' function tests: 1

    def test_list_permissions(self):
        """
        Test if it lists permissions for a vhost
        via rabbitmqctl list_permissions.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing stuff ...\nsaltstack\tsaltstack\t.*\t1\nguest\t0\tone\n...done",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_user_permissions("myuser"),
                {"saltstack": ["saltstack", ".*", "1"], "guest": ["0", "one"]},
            )

    # 'list_user_permissions' function tests: 1

    def test_list_user_permissions(self):
        """
        Test if it list permissions for a user
        via rabbitmqctl list_user_permissions.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "Listing stuff ...\nsaltstack\tsaltstack\t0\t1\nguest\t0\tone\n...done",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_user_permissions("myuser"),
                {"saltstack": ["saltstack", "0", "1"], "guest": ["0", "one"]},
            )

    # 'set_user_tags' function tests: 1

    def test_set_user_tags(self):
        """
        Test if it add user tags via rabbitmqctl set_user_tags.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.set_user_tags("myadmin", "admin"), {"Tag(s) set": "saltstack"}
            )

    # 'status' function tests: 1

    def test_status(self):
        """
        Test if it return rabbitmq status.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.status(), "saltstack")

    # 'cluster_status' function tests: 1

    def test_cluster_status(self):
        """
        Test if it return rabbitmq cluster_status.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.cluster_status(), "saltstack")

    # 'join_cluster' function tests: 1

    def test_join_cluster(self):
        """
        Test if it join a rabbit cluster.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.join_cluster("rabbit.example.com"), {"Join": "saltstack"}
            )

    # 'stop_app' function tests: 1

    def test_stop_app(self):
        """
        Test if it stops the RabbitMQ application,
        leaving the Erlang node running.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.stop_app(), "saltstack")

    # 'start_app' function tests: 1

    def test_start_app(self):
        """
        Test if it start the RabbitMQ application.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.start_app(), "saltstack")

    # 'reset' function tests: 1

    def test_reset(self):
        """
        Test if it return a RabbitMQ node to its virgin state
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.reset(), "saltstack")

    # 'force_reset' function tests: 1

    def test_force_reset(self):
        """
        Test if it forcefully Return a RabbitMQ node to its virgin state
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertEqual(rabbitmq.force_reset(), "saltstack")

    # 'list_queues' function tests: 1

    def test_list_queues(self):
        """
        Test if it returns queue details of the / virtual host
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "saltstack\t0\nceleryev.234-234\t10",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_queues(), {"saltstack": ["0"], "celeryev.234-234": ["10"]}
            )

    # 'list_queues_vhost' function tests: 1

    def test_list_queues_vhost(self):
        """
        Test if it returns queue details of specified virtual host.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "saltstack\t0\nceleryev.234-234\t10",
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.list_queues_vhost("consumers"),
                {"saltstack": ["0"], "celeryev.234-234": ["10"]},
            )

    # 'list_policies' function tests: 3

    def test_list_policies(self):
        """
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="3.7")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ), patch.dict(rabbitmq.__grains__, {"os_family": ""}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    def test_list_policies_freebsd(self):
        """
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="3.7")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ), patch.dict(rabbitmq.__grains__, {"os_family": "FreeBSD"}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    def test_list_policies_old_version(self):
        """
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="3.0")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ), patch.dict(rabbitmq.__grains__, {"os_family": ""}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    # 'set_policy' function tests: 1

    def test_set_policy(self):
        """
        Test if it set a policy based on rabbitmqctl set_policy.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.set_policy("/", "HA", ".*", '{"ha-mode": "all"}'),
                {"Set": "saltstack"},
            )

    # 'delete_policy' function tests: 1

    def test_delete_policy(self):
        """
        Test if it delete a policy based on rabbitmqctl clear_policy.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertDictEqual(
                rabbitmq.delete_policy("/", "HA"), {"Deleted": "saltstack"}
            )

    # 'policy_exists' function tests: 1

    def test_policy_exists(self):
        """
        Test if it return whether the policy exists
        based on rabbitmqctl list_policies.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="3.0")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ), patch.dict(rabbitmq.__grains__, {"os_family": ""}):
            self.assertFalse(rabbitmq.policy_exists("/", "HA"))

    # 'list_available_plugins' function tests: 2

    def test_list_available_plugins(self):
        """
        Test if it returns a list of plugins.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "saltstack\nsalt\nother",
                "stderr": "",
            }
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertListEqual(
                rabbitmq.list_available_plugins(), ["saltstack", "salt", "other"]
            )

    def test_list_available_plugins_space_delimited(self):
        """
        Test if it returns a list of plugins.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack salt other", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertListEqual(
                rabbitmq.list_available_plugins(), ["saltstack", "salt", "other"]
            )

    # 'list_enabled_plugins' function tests: 2

    def test_list_enabled_plugins(self):
        """
        Test if it returns a list of plugins.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "saltstack\nsalt\nother",
                "stderr": "",
            }
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertListEqual(
                rabbitmq.list_enabled_plugins(), ["saltstack", "salt", "other"]
            )

    def test_list_enabled_plugins_space_delimited(self):
        """
        Test if it returns a list of plugins.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack salt other", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertListEqual(
                rabbitmq.list_enabled_plugins(), ["saltstack", "salt", "other"]
            )

    # 'plugin_is_enabled' function tests: 2

    def test_plugin_is_enabled(self):
        """
        Test if it returns true for an enabled plugin.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": "saltstack\nsalt\nother",
                "stderr": "",
            }
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertTrue(rabbitmq.plugin_is_enabled("saltstack"))
            self.assertTrue(rabbitmq.plugin_is_enabled("salt"))
            self.assertTrue(rabbitmq.plugin_is_enabled("other"))

    def test_plugin_is_enabled_negative(self):
        """
        Test if it returns false for a disabled plugin.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack\nother", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertFalse(rabbitmq.plugin_is_enabled("salt"))
            self.assertFalse(rabbitmq.plugin_is_enabled("stack"))
            self.assertFalse(rabbitmq.plugin_is_enabled("random"))

    # 'enable_plugin' function tests: 1

    def test_enable_plugin(self):
        """
        Test if it enable a RabbitMQ plugin via the rabbitmq-plugins command.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertDictEqual(
                rabbitmq.enable_plugin("salt"), {"Enabled": "saltstack"}
            )

    # 'disable_plugin' function tests: 1

    def test_disable_plugin(self):
        """
        Test if it disable a RabbitMQ plugin via the rabbitmq-plugins command.
        """
        mock_run = MagicMock(
            return_value={"retcode": 0, "stdout": "saltstack", "stderr": ""}
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertDictEqual(
                rabbitmq.disable_plugin("salt"), {"Disabled": "saltstack"}
            )

    # 'list_upstreams' function tests: 1

    def test_list_upstreams(self):
        """
        Test if it returns a list of upstreams.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": 'federation-upstream\tremote-name\t{"ack-mode":"on-confirm"'
                ',"max-hops":1,"trust-user-id":true,"uri":"amqp://username:'
                'password@remote.fqdn"}',
                "stderr": "",
            }
        )
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            rabbitmq.__salt__, {"cmd.run_all": mock_run, "pkg.version": mock_pkg}
        ):
            self.assertDictEqual(
                rabbitmq.list_upstreams(),
                {
                    "remote-name": (
                        '{"ack-mode":"on-confirm","max-hops":1,'
                        '"trust-user-id":true,"uri":"amqp://username:'
                        'password@remote.fqdn"}'
                    )
                },
            )

    # 'upstream_exists' function tests: 2

    def test_upstream_exists(self):
        """
        Test whether a given rabbitmq-internal upstream exists based
        on rabbitmqctl list_upstream.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": 'federation-upstream\tremote-name\t{"ack-mode":"on-confirm"'
                ',"max-hops":1,"trust-user-id":true,"uri":"amqp://username:'
                'password@remote.fqdn"}',
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertTrue(rabbitmq.upstream_exists("remote-name"))

    def test_upstream_exists_negative(self):
        """
        Negative test of whether rabbitmq-internal upstream exists based
        on rabbitmqctl list_upstream.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": 'federation-upstream\tremote-name\t{"ack-mode":"on-confirm"'
                ',"max-hops":1,"trust-user-id":true,"uri":"amqp://username:'
                'password@remote.fqdn"}',
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertFalse(rabbitmq.upstream_exists("does-not-exist"))

    # 'add_upstream' function tests: 1

    def test_set_upstream(self):
        """
        Test if a rabbitMQ upstream gets configured properly.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": (
                    'Setting runtime parameter "federation-upstream" for component '
                    '"remote-name" to "{"trust-user-id": true, "uri": '
                    '"amqp://username:password@remote.fqdn", "ack-mode": "on-confirm", '
                    '"max-hops": 1}" in vhost "/" ...'
                ),
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertTrue(
                rabbitmq.set_upstream(
                    "remote-name",
                    "amqp://username:password@remote.fqdn",
                    ack_mode="on-confirm",
                    max_hops=1,
                    trust_user_id=True,
                )
            )

    # 'delete_upstream' function tests: 2

    def test_delete_upstream(self):
        """
        Test if an upstream gets deleted properly using rabbitmqctl delete_upstream.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 0,
                "stdout": (
                    'Clearing runtime parameter "remote-name" for component '
                    '"federation-upstream" on vhost "/" ...'
                ),
                "stderr": "",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertTrue(rabbitmq.delete_upstream("remote-name"))

    def test_delete_upstream_negative(self):
        """
        Negative test trying to delete a non-existant upstream.
        """
        mock_run = MagicMock(
            return_value={
                "retcode": 70,
                "stdout": (
                    'Clearing runtime parameter "remote-name" for component '
                    '"federation-upstream" on vhost "/" ...'
                ),
                "stderr": "Error:\nParameter does not exist",
            }
        )
        with patch.dict(rabbitmq.__salt__, {"cmd.run_all": mock_run}):
            self.assertRaises(
                CommandExecutionError, rabbitmq.delete_upstream, "remote-name"
            )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import salt.utils.platform
from tests.support.case import SSHCase
from tests.support.helpers import slowTest
from tests.support.unit import skipIf


@skipIf(salt.utils.platform.is_windows(), "salt-ssh not available on Windows")
class SSHSaltcheckTest(SSHCase):
    """
    testing saltcheck with salt-ssh
    """

    @slowTest
    def test_saltcheck_run_test(self):
        """
        test saltcheck.run_test with salt-ssh
        """
        saltcheck_test = {
            "module_and_function": "test.echo",
            "assertion": "assertEqual",
            "expected-return": "Test Works",
            "args": ["Test Works"],
        }
        ret = self.run_function("saltcheck.run_test", test=saltcheck_test)
        self.assertDictContainsSubset({"status": "Pass"}, ret)

    @slowTest
    def test_saltcheck_state(self):
        """
        saltcheck.run_state_tests
        """
        saltcheck_test = "validate-saltcheck"
        ret = self.run_function("saltcheck.run_state_tests", [saltcheck_test])
        self.assertDictContainsSubset(
            {"status": "Pass"}, ret[0]["validate-saltcheck"]["echo_test_hello"]
        )



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.runners.bgp as bgp

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf


@skipIf(not bgp.HAS_NAPALM, "napalm module required for this test")
class BGPTest(TestCase, LoaderModuleMockMixin):
    """
    Test the bgp runner
    """

    def setup_loader_modules(self):
        return {
            bgp: {
                "__opts__": {
                    "optimization_order": [0, 1, 2],
                    "renderer": "yaml",
                    "renderer_blacklist": [],
                    "renderer_whitelist": [],
                }
            }
        }

    def test_neighbors(self):
        ret = bgp.neighbors()
        self.assertEqual(ret, [])



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import os

import salt.pillar.saltclass as saltclass
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase


class SaltclassPillarTestCase(TestCase, LoaderModuleMockMixin):
    """
    Tests for salt.pillar.saltclass
    """

    def setup_loader_modules(self):
        return {saltclass: {}}

    def _runner(self, expected_ret):
        fake_args = {
            "path": os.path.abspath(
                os.path.join(RUNTIME_VARS.FILES, "saltclass", "examples")
            )
        }
        fake_pillar = {}
        fake_minion_id = "fake_id"
        try:
            full_ret = saltclass.ext_pillar(fake_minion_id, fake_pillar, fake_args)
            parsed_ret = full_ret["__saltclass__"]["classes"]
        # Fail the test if we hit our NoneType error
        except TypeError as err:
            self.fail(err)
        # Else give the parsed content result
        self.assertListEqual(expected_ret, parsed_ret)

    def test_succeeds(self):
        ret = [
            "default.users",
            "default.motd",
            "default.empty",
            "default",
            "roles.app",
            "roles.nginx",
        ]
        self._runner(ret)


class SaltclassPillarTestCaseListExpansion(TestCase, LoaderModuleMockMixin):
    """
    Tests for salt.pillar.saltclass variable expansion in list
    """

    def setup_loader_modules(self):
        return {saltclass: {}}

    def _runner(self, expected_ret):
        full_ret = {}
        parsed_ret = []
        fake_args = {
            "path": os.path.abspath(
                os.path.join(RUNTIME_VARS.FILES, "saltclass", "examples")
            )
        }
        fake_pillar = {}
        fake_minion_id = "fake_id"
        try:
            full_ret = saltclass.ext_pillar(fake_minion_id, fake_pillar, fake_args)
            parsed_ret = full_ret["test_list"]
        # Fail the test if we hit our NoneType error
        except TypeError as err:
            self.fail(err)
        # Else give the parsed content result
        self.assertListEqual(expected_ret, parsed_ret)

    def test_succeeds(self):
        ret = [{"a": "192.168.10.10"}, "192.168.10.20"]
        self._runner(ret)
